package jdk.management.resource.internal.inst;

import sun.misc.JavaIOFileDescriptorAccess;
import sun.misc.SharedSecrets;
import jdk.management.resource.ResourceRequest;
import jdk.management.resource.ResourceRequestDeniedException;
import java.io.FileNotFoundException;
import jdk.management.resource.internal.ApproverGroup;
import jdk.management.resource.internal.ResourceIdImpl;
import jdk.internal.instrumentation.InstrumentationMethod;
import java.io.IOException;
import java.io.FileDescriptor;
import jdk.internal.instrumentation.InstrumentationTarget;

@InstrumentationTarget("java.io.FileInputStream")
public final class FileInputStreamRMHooks
{
    private final FileDescriptor fd;
    private String path;
    private final Object closeLock;
    private volatile boolean closed;
    
    public FileInputStreamRMHooks() {
        this.fd = null;
        this.path = null;
        this.closeLock = new Object();
        this.closed = false;
    }
    
    @InstrumentationMethod
    public final FileDescriptor getFD() throws IOException {
        return this.getFD();
    }
    
    @InstrumentationMethod
    private void open(String s) throws FileNotFoundException {
        ResourceIdImpl of = ResourceIdImpl.of(this.path);
        ResourceRequest approver = ApproverGroup.FILE_OPEN_GROUP.getApprover(this);
        long request;
        try {
            request = approver.request(1L, of);
            if (request < 1L) {
                throw new FileNotFoundException(s + ": resource limited: too many open files");
            }
        }
        catch (ResourceRequestDeniedException cause) {
            FileNotFoundException ex = new FileNotFoundException(s + ": resource limited: too many open files");
            ex.initCause(cause);
            throw ex;
        }
        ResourceRequest approver2 = null;
        long request2 = 0L;
        int n = 0;
        try {
            FileDescriptor fd = null;
            try {
                fd = this.getFD();
            }
            catch (IOException ex2) {}
            approver2 = ApproverGroup.FILEDESCRIPTOR_OPEN_GROUP.getApprover(fd);
            try {
                request2 = approver2.request(1L, of);
                if (request2 < 1L) {
                    throw new FileNotFoundException(s + ": resource limited: too many open file descriptors");
                }
            }
            catch (ResourceRequestDeniedException cause2) {
                FileNotFoundException o = new FileNotFoundException(s + ": resource limited: too many open file descriptors");
                o.initCause(cause2);
                throw o;
            }
            this.open(s);
            n = 1;
        }
        finally {
            approver2.request(-(request2 - (long)n), of);
            approver.request(-(request - (long)n), of);
        }
    }
    
    @InstrumentationMethod
    public int read() throws IOException {
        ResourceIdImpl of = ResourceIdImpl.of(this.path);
        ResourceRequest resourceRequest;
        if (this.getFD() == FileDescriptor.in) {
            resourceRequest = ApproverGroup.STDIN_READ_GROUP.getApprover(this);
        }
        else {
            resourceRequest = ApproverGroup.FILE_READ_GROUP.getApprover(this);
        }
        long request;
        try {
            request = resourceRequest.request(1L, of);
            if (request < 1L) {
                throw new IOException("Resource limited: insufficient bytes approved");
            }
        }
        catch (ResourceRequestDeniedException cause) {
            throw new IOException("Resource limited", cause);
        }
        int read = -1;
        try {
            read = this.read();
        }
        finally {
            resourceRequest.request(-(request - (long)((read == -1) ? false : true)), of);
        }
        return read;
    }
    
    @InstrumentationMethod
    public int read(byte[] array) throws IOException {
        if (array == null) {
            return this.read(array);
        }
        ResourceIdImpl of = ResourceIdImpl.of(this.path);
        ResourceRequest resourceRequest;
        if (this.getFD() == FileDescriptor.in) {
            resourceRequest = ApproverGroup.STDIN_READ_GROUP.getApprover(this);
        }
        else {
            resourceRequest = ApproverGroup.FILE_READ_GROUP.getApprover(this);
        }
        int length = array.length;
        long max;
        try {
            max = Math.max(resourceRequest.request((long)length, of), 0L);
        }
        catch (ResourceRequestDeniedException cause) {
            throw new IOException("Resource limited", cause);
        }
        int n = 0;
        int n2 = 0;
        try {
            if (max < (long)length) {
                resourceRequest.request(-max, of);
                n2 = this.read(array, 0, array.length);
                n = Math.max(n2, 0);
            }
            else {
                n2 = this.read(array);
                n = Math.max(n2, 0);
            }
        }
        finally {
            resourceRequest.request(-(max - (long)n), of);
        }
        return n2;
    }
    
    @InstrumentationMethod
    public int read(byte[] array, int n, int min) throws IOException {
        if (min < 0) {
            return this.read(array, n, min);
        }
        ResourceIdImpl of = ResourceIdImpl.of(this.path);
        ResourceRequest resourceRequest;
        if (this.getFD() == FileDescriptor.in) {
            resourceRequest = ApproverGroup.STDIN_READ_GROUP.getApprover(this);
        }
        else {
            resourceRequest = ApproverGroup.FILE_READ_GROUP.getApprover(this);
        }
        long max;
        try {
            max = Math.max(resourceRequest.request((long)min, of), 0L);
        }
        catch (ResourceRequestDeniedException cause) {
            throw new IOException("Resource limited", cause);
        }
        min = Math.min(min, (int)max);
        int max2 = 0;
        int read;
        try {
            read = this.read(array, n, min);
            max2 = Math.max(read, 0);
        }
        finally {
            resourceRequest.request(-(max - (long)max2), of);
        }
        return read;
    }
    
    @InstrumentationMethod
    public void close() throws IOException {
        synchronized (this.closeLock) {
            if (this.closed) {
                return;
            }
        }
        JavaIOFileDescriptorAccess javaIOFileDescriptorAccess = SharedSecrets.getJavaIOFileDescriptorAccess();
        long handle;
        try {
            handle = javaIOFileDescriptorAccess.getHandle(this.fd);
            if (handle == -1L) {
                handle = (long)javaIOFileDescriptorAccess.get(this.fd);
            }
        }
        catch (UnsupportedOperationException ex) {
            handle = (long)javaIOFileDescriptorAccess.get(this.fd);
        }
        try {
            this.close();
            long handle2;
            try {
                handle2 = javaIOFileDescriptorAccess.getHandle(this.fd);
                if (handle2 == -1L) {
                    handle2 = (long)javaIOFileDescriptorAccess.get(this.fd);
                }
            }
            catch (UnsupportedOperationException ex2) {
                handle2 = (long)javaIOFileDescriptorAccess.get(this.fd);
            }
            ResourceIdImpl of = ResourceIdImpl.of(this.path);
            if (handle2 != handle) {
                ApproverGroup.FILEDESCRIPTOR_OPEN_GROUP.getApprover(this.fd).request(-1L, of);
            }
            ApproverGroup.FILE_OPEN_GROUP.getApprover(this).request(-1L, of);
        }
        finally {
            long handle3;
            try {
                handle3 = javaIOFileDescriptorAccess.getHandle(this.fd);
                if (handle3 == -1L) {
                    handle3 = (long)javaIOFileDescriptorAccess.get(this.fd);
                }
            }
            catch (UnsupportedOperationException ex3) {
                handle3 = (long)javaIOFileDescriptorAccess.get(this.fd);
            }
            ResourceIdImpl of2 = ResourceIdImpl.of(this.path);
            if (handle3 != handle) {
                ApproverGroup.FILEDESCRIPTOR_OPEN_GROUP.getApprover(this.fd).request(-1L, of2);
            }
            ApproverGroup.FILE_OPEN_GROUP.getApprover(this).request(-1L, of2);
        }
    }
}
